import threading

import discord
from discord.ext import tasks, commands

from src.config_dict import ConfigDict
from src.steam_service import get_server_list, get_populated_servers


from src.configuration import Configuration
from src.roles import Roles


class NTPubBot(discord.ext.commands.Bot):
    def __init__(self, *args, **options):
        super().__init__(*args, **options)

        self.config = ConfigDict()

        self.lock_emojis = {
            # Keys are the boolean int values returned by "password_protected"
            1: discord.PartialEmoji(name=":lock:"),
            0: discord.PartialEmoji(name=":unlock:")
        }

        self.lock = threading.Lock()
        self.fetch_servers_info.start()

    async def on_ready(self):
        print('Logged in as')
        print(self.user.name)
        print(self.user.id)
        print('------')

    @tasks.loop(seconds=1.0)
    async def fetch_servers_info(self):
        with self.lock:
            servers = get_server_list()
            populated_servers = get_populated_servers(servers, self.config["min_pop"])

            if len(populated_servers) > 0:
                await self.send_channel_notification(populated_servers)

        self.timeout_fetch_task()

    async def send_channel_notification(self, populated_servers):
        channel = self.get_channel(self.config["channel_id"])
        role = channel.guild.get_role(self.config["role_id"])

        message = f"{role.mention}\n People are playing on:\n\n"
        for server in populated_servers:
            server_protected_emoji = self.lock_emojis[server["password_protected"]]
            message += f'{server["server_name"]} {server_protected_emoji} - {server["map"]} \n'
            message += f'Players: {server["player_count"]} / {server["max_players"]} \n\n'

        await channel.send(message)

    @fetch_servers_info.before_loop
    async def before_fetch_task(self):
        await self.wait_until_ready()

    def timeout_fetch_task(self):
        self.fetch_servers_info.stop()
        timer = threading.Timer(float(self.config["timeout"]), self.restart_fetch_task)
        timer.start()

    def restart_fetch_task(self):
        self.fetch_servers_info.start()


def main():
    intents = discord.Intents.default()
    intents.members = True

    bot = NTPubBot(intents=intents, command_prefix="!")
    config_cog = Configuration(bot)
    roles_cog = Roles(bot, config_cog.config["role_id"],config_cog.config["message_id"],
                      config_cog.config["channel_id"], config_cog.config["emoji_id"])
    bot.add_cog(config_cog)
    bot.add_cog(roles_cog)

    bot.run(bot.config["discord_token"])


if __name__ == "__main__":
    main()
